; Realisierung einer Menanzeige mit Wandlung des Ergebis des A/D-Wandlers und Anzeige im Display als
; Festkommazahl
; Quellcode wurde fr einen ph-Wert- Controller (Aquarium) vorbestimmt
; Eine Zweipunktregelung wurde noch nicht implementiert.

.NOLIST
.include "m8def.inc"
.LIST

; Definitionen der Arbeitsregister, Register zur Berechnung (R0..R15) werden nicht definiert, da nur
; temporr mit realen Namen benutzt
; Register R26..R31 bleiben Pointerregistern vorbehalten
 
.def	rmp 		= R16	; Arbeitsregister
.def	KeyStatus	= R17	; Abbild der eingelesenen Tasten am Eingangsport
.def	Merker 		= R18	; Zwischenspeicher bei Tastenabfrage -> Tasten eingeschwungen?
.def	temp 		= R19	; Arbeitsregister fr Interrupts
.def	temp2 		= R20	; reserviert fr dcf77dec-Modul
.def	temp3		= R21	; reserviert fr dcf77dec-Modul
.def	temp4 		= R22	; reserviert fr dcf77dec-Modul
.def	temp5 		= R23	; reserviert fr dcf77dec-Modul
.def	ZaehlerL	= R24	; reserviert fr 16-bit- Zhler (unbenutzt)
.def	ZaehlerH	= R25	; reserviert fr 16-bit- Zhler (unbenutzt)

; Definitionen der Speicherzellen im SRAM
.equ	PrellenOk		= 0x0060 	; Zwichenspeicher Tastenabfrage -> erste Speicherzelle im SRAM
.equ 	WertL 			= 0x0061 	; Wertregister low Analogwert
.equ 	WertH 			= 0x0062 	; Wertregister high Analogwert
.equ	Zehner			= 0x0063 	; Speicherzelle im SRAM: Analogwert Zehner
.equ	Einer			= 0x0064 	; Speicherzelle im SRAM: Analogwert Einer
.equ	Punkt			= 0x0065 	; Speicherzelle im SRAM: Analogwert Trennzeichen
.equ	Zehntel			= 0x0066 	; Speicherzelle im SRAM: Analogwert Zehntel
.equ	Hundertstel		= 0x0067 	; Speicherzelle im SRAM: Analogwert Hundertstel
.equ	Term			= 0x0068 	; Speicherzelle im SRAM: Analogwert Terminierung
.equ	Menue			= 0x0069	; Wert, welches Men angezeigt werden soll
.equ	Menuemerker		= 0x006A	; Merker fr Menanzeige
.equ	SchaltwertL		= 0x006B 	; ph-Schaltwert low
.equ	SchaltwertH		= 0x006C 	; ph-Schaltwert high
.equ	ZaehlerTasteMehr	= 0x006D	; Zhler fr "langsame" Durchlufe Taste Mehr
.equ	ZaehlerTasteWeniger	= 0x006E	; Zhler fr "langsame" Durchlufe Taste Weniger

; Definitionen E/A
.equ key_port 	  = PinD	; Festlegung des Tasteneingangsports
.equ out_port_ddr = DDRC	; Datenrichtungsregister fr Ausgnge
.equ out_port 	  = PortC	; Ausgnge

; Definitionen fr LCD_Basisroutine.asm
.equ pLcdPort = PortB		; Ausgangsport fr LCD-Anzeige ist PortB
.equ pLcdDdr = DDRB			; Datenrichtungsregister fr LCD-Anzeige ist DDRB
.equ ftakt = 8000000		; 8MHz - Systemtakt

.CSEG
.ORG $000

;Interrupteinsprungadressen definieren
		rjmp INITIAL	;RESET		-	Bearbeitung
		reti			;INT0		-	externer Interrupt_0
		reti			;INT1		-	externer Interrupt_1
		reti			;TIM2_COMP	-	TIMER2 COMPARE
		reti			;TIM2_OVF	-	TIMER2 OVERVLOW
		reti			;TIM1_CAPT	-	TIMER1 CAPTURE
		reti			;TIM1_COMPA	-	TIMER1 COMPARE_A
		reti			;TIM1_COMPB	-	TIMER1 COMPARE_B
		rjmp TIMER_1	;TIM1_OVF	-	TIMER1 Overflow
		rjmp TIMER_0	;TIM0_OVF	-	TIMER0 Overflow
		reti			;SPI		-	SPI Work beendet
		reti			;UART_RX	-	RX_Byte empfangen
		reti			;UART_DRE	-	UDRE_Datenregister leer
		reti			;UART_TX	-	TX_Work beendet
		reti			;ADC		-	analog-digital Wandlung
		reti			;EE_RDY		-	EEPROM ready
		reti			;ANA_COMP	-	Analog Komparator
		reti			;TWSI		-	Two-Wire Serial Interface
		reti			;SPM_RDY	-	Store Program Memory Ready

; Interruptroutinen
TIMER_0:	ldi temp,0x00
			out TCCR0, temp		; Stoppen des Timers TC0
			ldi temp, 0x01
			sts PrellenOk, temp	; Testenprellmerker setzen
			reti
								; zurck aus Interrupt und erneute Freigabe Interrupts
TIMER_1:	ldi temp,0x00
			out TCCR1B, temp		; Stoppen des Timers TC1
			rcall dcf77_start
			ldi temp, 0x04
			out tcnt1H, temp
			ldi temp, 0xE2
			out tcnt1L, temp		; Startwert 1250 dezimal in Timer1 eintragen (10ms bei 8MHz und Teiler 64)
			ldi temp, 0b00000011
			out TCCR1B, temp		; Timer1 mit Teiler 64 starten
			reti

; AVR Initialisierung
INITIAL:
			ldi rmp, high(RAMEND)
			out SPH, rmp
			ldi rmp, low(RAMEND)
			out	SPL, rmp			; Stackpointer initialisieren

			rcall Lcd4Init     		; Display initialisieren und Anzeige lschen, Festlegung des Displayports

; Register und Speicherzellen initialisieren
			ldi Merker,0b00111100	; Startwert Register fr Tastenmerker

			ldi rmp, 0x00
			sts PrellenOk, rmp		; Startwert fr Testenprellmerker
						
			ldi rmp, 0				; Menwahl Vorgabe laden
			sts Menue, rmp

			ldi rmp, 0xF5			; Startwerte fr Tastenzhler in SRAM laden
			sts ZaehlerTasteMehr, rmp
			sts ZaehlerTasteWeniger, rmp
			
			ldi rmp, 0x00			; Menmerker initialisieren -> EEPROM-Werte nur einmalig lesen
			sts Menuemerker, rmp

; Ports initialisieren
			ldi	rmp, 0b11111100
			out	key_port, rmp		; 0b11111100 in PortD schaltet teilweise einen PullUp-Widerstand ein, der bei
									; offenem Eingang undefinierte Zustnde vermeidet
			
			ldi rmp, 0b11111110       	
        	out out_port_ddr, rmp   ; einige Pin vom E/A-Port sind jetzt Ausgnge durch Ausgabe 
									; in Datenrichtungs-Register

			cbi out_port, 4	 		; Bit4 in out_port auf 0 setzen
			cbi out_port, 5	 		; Bit5 in out_port auf 0 setzen
			
; A/D-Wandler initialisieren
			ldi rmp,0x00
			out ADMUX,rmp		;Pin0 von PortC als Analogeingang im MUX setzen
			ldi rmp,0b11100111
			out ADCSR,rmp		;A/D-Wandler ein, Freemode, Interrupt nicht aktiv, Teiler=128

; Funkuhrmodul dcf77 initialisieren
			ldi rmp, 0
			sts DCF77CT, rmp
			sts DCF77CHECK, rmp
			sts DCF77STATE, rmp
			ldi ZH, high(NEWTIME)
			ldi ZL, low(NEWTIME)
			st Z+, rmp
			st Z+, rmp
			st Z+, rmp
			st Z+, rmp
			st Z+, rmp
			st Z+, rmp
			ldi rmp, 1
			sts DCF77SHIFT, rmp
			rcall dcf77_init

; Intro LCD abspielen
			rcall intro

; Timer initialisieren fr dcf77-Modul
			ldi rmp, 0x04
			out tcnt1H, rmp
			ldi rmp, 0xE2
			out tcnt1L, rmp		; Startwert 1250 dezimal in Timer1 eintragen (10ms bei 8MHz und Teiler 64)
			ldi rmp,0b00000101
			out TIMSK,rmp		;TC0, TC1 overflowInterrupt freigeben -> beim AT4433 ist TC0 Bit1
			ldi rmp, 0b00000011
			out TCCR1B, rmp		; Timer1 mit Teiler 64 starten

; Ende der Initialisierung
			sei					; globale Interruptfreigabe
			rjmp HAUPT			

; Unterprogramme
weniger:	lds	R5, ZaehlerTasteWeniger ; Zhler fr "langsame" Durchlufe laden
			lds R1, SchaltwertL
			lds R2, SchaltwertH	; Schaltwert aus SRAM laden
			ldi rmp, 0x01
			mov R3, rmp
			ldi rmp, 0x00
			mov R4, rmp			; Subtrahenden 16bit fr Schaltwert laden
        	sub R1, R3
        	sbc R2, R4			; Schaltwert 16bit verringern
           	brmi weniger1				; zu weniger1: springen, wenn Ergebnis negativ
										; -> nicht speichern 
			sts SchaltwertH, R2		
			sts SchaltwertL, R1			; Schaltwert speichern
weniger1:	inc R5
			breq weniger2				; grere Wartezeit fr Durchlauf berspringen
			sts ZaehlerTasteWeniger, R5	; Zhler fr "langsame" Durchlufe speichern
			rcall Delay400ms
			ret
weniger2:	rcall Delay50ms
			ret

mehr:		lds R3, ZaehlerTasteMehr	; Zhler fr "langsame" Durchlufe laden
			lds R1, SchaltwertL
			lds R2, SchaltwertH	; aktuellen Schaltwert laden
			inc R1				; SchaltwertL um 1 erhhen
			brne mehr1			; wenn kein berlauf stattfndet, dann zu mehr1:
			inc R2				; SchaltwertH um 1 erhhen
			ldi rmp, 0xFC	
			add rmp, R2			; high Endwert fr A/D- Wandler mit 0xFC zwecks berlauftest
								; addieren -> berlauf dann bei 0x03FF des 16bit Schaltwertes
			brcs mehr2			; wenn c-Flag (berlauf) gesetzt wurde, dann springe zu mehr2: -> nicht speichern
mehr1:		sts SchaltwertH, R2		
			sts SchaltwertL, R1	; Schaltwert speichern
mehr2:		inc R3
			breq mehr3
			sts ZaehlerTasteMehr, R3	; Zhler fr "langsame" Durchlufe speichern
			rcall Delay400ms
			ret
mehr3:		rcall Delay50ms
			ret

Menuewahl:	lds rmp, Menue		; alte Menauswahl aus SRAM laden
			dec rmp				; Menwahl um 1 mindern
			brmi Menuewahl1		; Verzweige wenn negativer Wert
			sts Menue, rmp		; neue Menauswahl im SRAM abspeichern
			rcall Delay400ms	; nach Tastendruck 400ms warten
			rcall Lcd4Clear		; LCD-Anzeige lschen
			ret
Menuewahl1:
			ldi rmp, 5			; Platz fr 6 Menpunkte
			sts Menue, rmp		; neue Menauswahl im SRAM abspeichern
			rcall Delay400ms	; nach Tastendruck 400ms warten
			rcall Lcd4Clear		; LCD-Anzeige lschen
			ret

Wandlung:	rcall fpconv10		; Wert dezimal im SRAM ablegen
			sts Zehner, R5
			sts Einer, R6
			sts Punkt, R7
			sts Zehntel, R8
			sts Hundertstel, R9
			sts	Term, R10
			ret

; Verzgerung um 400 ms fr den Tastendruck -> bentigt LCD_Basisroutine
Delay400ms:
			ldi rmp,80			; 80 * 5 ms warten
Delay400ms1:
			push rmp 			; Rette Byte auf Stapel
			rcall LcdDelay5ms
			pop rmp 			; Stelle Byte wieder her
			dec rmp
			brne Delay400ms1
			ret					; Fertig

; Verzgerung um 50 ms fr den Tastendruck (Schnelldurchlauf) -> bentigt LCD_Basisroutine
Delay50ms:
			ldi rmp,10			; 10 * 5 ms warten
Delay50ms1:
			push rmp 			; Rette Byte auf Stapel
			rcall LcdDelay5ms
			pop rmp 			; Stelle Byte wieder her
			dec rmp
			brne Delay50ms1
			ret					; Fertig

; Routine zum Lesen von 16-bit EE-Prom Werten
; rmp - bergabewert der EEPROM-Adresse, R1 und R2 beinhalten die Werte des EEPROMs
EEPROM_Read:
		sbic EECR, EEWE			
		rjmp EEPROM_Read		; Warteschleife, bis EEPROM bereit zum Lesen
        out EEARH, ZH
		out EEARL, ZL			; EEPROM-Adresse
		sbi EECR, EERE          ; Lesevorgang starten
        in rmp, EEDR            ; gelesenes Byte nach "rmp" schreiben
        mov R2, rmp             ; Inhalt rmp an R2 bergeben (Highbyte Daten)
        adiw ZL, 1				; gesamten Zeiger Z um 1 erhhen
EEPROM_Read1:
		sbic EECR, EEWE			
		rjmp EEPROM_Read1		; Warteschleife, bis EEPROM bereit zum Lesen
		out EEARH, ZH
		out EEARL, ZL			; EEPROM-Adresse
		sbi EECR, EERE          ; Lesevorgang starten
        in rmp, EEDR            ; gelesenes Byte nach "rmp"
        mov R1, rmp             ; Inhalt rmp an R1 bergeben (Lowbyte Daten)
        ret						; Ende der Funktion und Rcksprung

; Routine zum Schreiben von 16-bit EE-Prom Werten
; rmp - bergabewert der EEPROM-Adresse, R1 und R2 beinhalten die Werte fr den EEPROM
EEPROM_Write:
		sbic EECR, EEWE			
		rjmp EEPROM_Write		; Warteschleife, bis EEPROM bereit zum Schreiben
        out EEARH, ZH
		out EEARL, ZL			; EEPROM-Adresse
		mov rmp, R2             ; Inhalt R2 an rmp bergeben (Highbyte Daten)
		out EEDR, rmp           ; Byte von "rmp" nach Datenregister EEPROM schreiben
		sbi EECR, EEMWE         ; Schreibvorgang vorbereiten
		sbi EECR, EEWE			; Schreibvorgang starten
        adiw ZL, 1				; gesamten Zeiger Z um 1 erhhen
EEPROM_Write1:
		sbic EECR, EEWE			
		rjmp EEPROM_Write1		; Warteschleife, bis EEPROM bereit zum Lesen
		out EEARH, ZH
		out EEARL, ZL			; EEPROM-Adresse
		mov rmp, R1             ; Inhalt R1 an rmp bergeben (Lowbyte Daten)
		out EEDR, rmp           ; Byte von "rmp" nach Datenregister EEPROM schreiben
		sbi EECR, EEMWE         ; Schreibvorgang vorbereiten
		sbi EECR, EEWE			; Schreibvorgang starten
        ret						; Ende der Funktion und Rcksprung

; Routine zum Entprellen der Tasteneingnge durch Einfgen von einer Warteschleife
ENTPRELLEN:	mov Merker, KeyStatus	; Kopieren Status Tasten
			ldi rmp ,0b00000101
			out TCCR0, rmp			; Start des Timers TC0 mit Takt/1024
SCHLEIFE:	lds rmp, PrellenOk		; Testenprellmerker laden
			sbrs rmp, 0				; berspringe nchste Zeile, wenn Bit 0 in rmp gesetzt ist
			rjmp SCHLEIFE			; Warten auf Interrupt TC0 wegen Tastenprellen
			ldi rmp, 0x00
			sts PrellenOk, rmp		; Testenprellmerker rckseten
			rjmp HAUPT

; Hauptprogramm		
HAUPT:		in rmp,ADCL
			sts WertL, rmp
			in rmp,ADCH
			sts WertH, rmp		; aktuellen ph-Wert im SRAM ablegen

			sbi out_port, 4	 	; Bit4 in out_port auf 1 setzen
			sbi out_port, 5	 	; Bit5 in out_port auf 1 setzen

			in KeyStatus, key_port		; Einlesen der Tasten in Register
			ori KeyStatus, 0b11000011	; Eingang key_port Bit 0,1,6,7 unbercksichtigt lassen
			cp Merker, KeyStatus		; Vergleichen ob Tasten vom Port eingeschwungen sind
			brne ENTPRELLEN				; Sprung zum entprellen, wenn Port nicht eingeschwungen

			sbis key_port, 2	; berspringe nchste Zeile, wenn Bit 2 gesetzt ist, da low aktiv
			rcall mehr

			ldi rmp, 0xF5				; Startwerte fr Tastenzhler in SRAM laden
			sbic key_port, 2			; Taste Mehr nicht bettigt, dann...
			sts ZaehlerTasteMehr, rmp	;  Zhler fr Schnelldurchlauf zurcksetzen

			sbis key_port, 3	; berspringe nchste Zeile, wenn Bit 3 gesetzt ist, da low aktiv
			rcall weniger

			ldi rmp, 0xF5					; Startwerte fr Tastenzhler in SRAM laden
			sbic key_port, 3				; Taste Weniger nicht bettigt, dann...
			sts ZaehlerTasteWeniger, rmp	; Zhler fr Schnelldurchlauf zurcksetzen
			
			sbis key_port, 4	; berspringe nchste Zeile, wenn Bit 4 gesetzt ist, da low aktiv
			rcall Menuewahl
			
			lds rmp, Menue		; Menauswahl mittels Z-Register
			ldi ZH, high(Menuetabelle)
			ldi ZL, low(Menuetabelle)
			add ZL, rmp
			brcc Ueberlauf
			inc ZH
Ueberlauf:	ijmp

Menuetabelle:
			rjmp phAnzeige		; Sprung zum Menpunkt Anzeige aktueller ph-Wert
			rjmp SchaltAusLuft	; Sprung zum Menpunkt Schaltpunkt Aus Luftpumpe
			rjmp SchaltAnLuft	; Sprung zum Menpunkt Schaltpunkt An Luftpumpe			
			rjmp SchaltAusMV	; Sprung zum Menpunkt Schaltpunkt Aus Magnetventil
			rjmp SchaltAnMV		; Sprung zum Menpunkt Schaltpunkt An Magnetventil
			rjmp Uhr			; Sprung zum Menpunkt Anzeige Uhrzeit
			rjmp Haupt			; Rcksprung zur Sicherheit

Uhr:		ldi rmp,0x84 		
			rcall Lcd4Ctrl		; Curser verschieben in 1.Zeile 5.Zeichen in LCD
			lds rmp, STU
			rcall Lcd4PBcd
			ldi rmp, ':'
			rcall Lcd4Chr
			lds rmp, MIN
			rcall Lcd4PBcd
			ldi rmp, ':'
			rcall Lcd4Chr
			lds rmp, SEK
			rcall Lcd4PBcd
			ldi rmp,0xC4 		
			rcall Lcd4Ctrl		; Curser verschieben in 2.Zeile 5.Zeichen in LCD
			lds rmp, TAG
			rcall Lcd4PBcd
			ldi rmp, '.'
			rcall Lcd4Chr
			lds rmp, MON
			rcall Lcd4PBcd
			ldi rmp, '.'
			rcall Lcd4Chr
			lds rmp, JHR
			rcall Lcd4PBcd
			ldi rmp,0xC0 		
			rcall Lcd4Ctrl		; Curser verschieben in 2.Zeile 1.Zeichen in LCD
			ldi rmp, 7			; Fehlerabfrage Wochentag
			lds R1, WTAG
			cp rmp, R1
			brcs ErrorWTag
			mov rmp, R1
			rcall Lcd4PBcd		; Wochentaganzeige mu noch erweitert werden
ErrorWTag:	ldi ZH, high(WTagError)
			ldi ZL, low(WTagError)
			rcall LcdEepPrint
			rjmp HAUPT

phAnzeige:	lds R1, WertL
			lds R2, WertH
			rcall Wandlung
			ldi rmp,0x83 		
			rcall Lcd4Ctrl		; Curser verschieben in 1.Zeile 4.Zeichen in LCD
        	ldi ZH, high(text1)
			ldi ZL, low(text1)	; Adresse des text1 von EEPROM laden
        	rcall LcdEepPrint	; Unterfunktion LcdEepPrint aufrufen
			ldi rmp,0xC0 		
			rcall Lcd4Ctrl		; Curser verschieben in 2.Zeile 1.Zeichen in LCD
			ldi ZH, high(text7)
			ldi ZL, low(text7)	; Adresse des text7 von EEPROM laden
        	rcall LcdEepPrint	; Unterfunktion LcdEepPrint aufrufen
			ldi rmp,0xC9
			rcall Lcd4Ctrl		; Curser verschieben in 2.Zeile 10.Zeichen in LCD
			ldi ZH, high(Zehner)
			ldi ZL, low(Zehner)	; Zeiger auf Adresse fr gewandelte Zahl -> Zehnerstelle setzen
								; Unterschlagung der Zehnerstelle mglich

			ldi rmp, 5			; bergabewert fr die Anzeige der 5 nchst folgenden Zeichen
								; aus SRAM
			rcall Lcd4RTxt
			rjmp HAUPT

SchaltAnMV:
			lds rmp, Menuemerker; letzten Menmerker laden, damit nur 1mal der Schaltwert fr
			sbrc rmp, 0			; Magnetventil An aus dem EEPROM in SRAM geladen werden
			rjmp SchaltAnMV1
			ldi rmp, 0b00000001
			sts Menuemerker, rmp; nochmaliges Laden aus EEPROM sperren
			ldi ZH, high(WertMV_An)
			ldi ZL, low(WertMV_An)	; Adresse des Schaltwertes MV An von EEPROM laden
			rcall EEPROM_Read	; aus EEPROM Schaltwert lesen
			sts SchaltwertH, R2
			sts SchaltwertL, R1	; ausgelesenen Schaltwert speichern
SchaltAnMV1:
			lds R1, SchaltwertL	; Schaltwert aktuell aus SRAM laden zur Vorbereitung anschlieender
			lds R2, SchaltwertH	; Wandlung
			rcall Wandlung		; Schaltwert in Fliekommazahl wandeln
			ldi rmp,0x81 		
			rcall Lcd4Ctrl		; Curser verschieben in 1.Zeile 2.Zeichen in LCD
			ldi ZH, high(text2)
			ldi ZL, low(text2)	; Adresse des text2 von EEPROM laden
			rcall LcdEepPrint	; Unterfunktion LcdEepPrint aufrufen
			ldi rmp,0xC0 		
			rcall Lcd4Ctrl		; Curser verschieben in 2.Zeile 1.Zeichen in LCD
			ldi ZH, high(text3)
			ldi ZL, low(text3)	; Adresse des text3 von EEPROM laden
			rcall LcdEepPrint	; Unterfunktion LcdEepPrint aufrufen
			ldi rmp,0xC9
			rcall Lcd4Ctrl		; Curser verschieben in 2.Zeile 10.Zeichen in LCD
			ldi ZH, high(Zehner)
			ldi ZL, low(Zehner)	; Zeiger auf Adresse fr gewandelte Zahl -> Zehnerstelle setzen
								; Unterschlagung der Zehnerstelle mglich

			ldi rmp, 5			; bergabewert fr die Anzeige der 5 nchst folgenden Zeichen
								; aus SRAM
			rcall Lcd4RTxt
			sbis key_port, 5	; berspringe nchste Zeile, wenn Bit 5 gesetzt ist, da low aktiv
			rcall SchaltAnMVstore
			rjmp HAUPT	
SchaltAnMVstore:
			ldi ZH, high(WertMV_An)
			ldi ZL, low(WertMV_An) ; bergabeparameter fr EEPROM Speicherung laden
			lds R1, SchaltwertL
			lds R2, SchaltwertH
			rcall EEPROM_Write	; Schaltwert Magnetventil An in EEPROM speichern
			rcall Delay400ms	; nach Tastendruck 400ms warten
			ret

SchaltAusMV:
			lds rmp, Menuemerker; letzten Menmerker laden, damit nur 1mal der Schaltwert fr
			sbrc rmp, 1			; Magnetventil Aus aus dem EEPROM in SRAM geladen werden
			rjmp SchaltAusMV1
			ldi rmp, 0b00000010
			sts Menuemerker, rmp; nochmaliges Laden aus EEPROM sperren
			ldi ZH, high(WertMV_Aus)
			ldi ZL, low(WertMV_Aus)
			rcall EEPROM_Read	; aus EEPROM Schaltwert lesen
			sts SchaltwertH, R2
			sts SchaltwertL, R1	; ausgelesenen Schaltwert speichern
SchaltAusMV1:
			lds R1, SchaltwertL	; Schaltwert aktuell aus SRAM laden zur Vorbereitung anschlieender
			lds R2, SchaltwertH	; Wandlung
			rcall Wandlung		; Schaltwert in Fliekommazahl wandeln
			ldi rmp,0x81 		
			rcall Lcd4Ctrl		; Curser verschieben in 1.Zeile 2.Zeichen in LCD
			ldi ZH, high(text2)
			ldi ZL, low(text2)	; Adresse des text2 von EEPROM laden
			rcall LcdEepPrint	; Unterfunktion LcdEepPrint aufrufen
			ldi rmp,0xC0 		
			rcall Lcd4Ctrl		; Curser verschieben in 2.Zeile 1.Zeichen in LCD
			ldi ZH, high(text4)
			ldi ZL, low(text4)	; Adresse des text4 von EEPROM laden
			rcall LcdEepPrint	; Unterfunktion LcdEepPrint aufrufen
			ldi rmp,0xC9
			rcall Lcd4Ctrl		; Curser verschieben in 2.Zeile 10.Zeichen in LCD
			ldi ZH, high(Zehner)
			ldi ZL, low(Zehner)	; Zeiger auf Adresse fr gewandelte Zahl -> Zehnerstelle setzen
								; Unterschlagung der Zehnerstelle mglich

			ldi rmp, 5			; bergabewert fr die Anzeige der 5 nchst folgenden Zeichen
								; aus SRAM
			rcall Lcd4RTxt
			sbis key_port, 5	; berspringe nchste Zeile, wenn Bit 5 gesetzt ist, da low aktiv
			rcall SchaltAusMVstore
			rjmp HAUPT	
SchaltAusMVstore:
			ldi ZH, high(WertMV_Aus)
			ldi ZL, low(WertMV_Aus)	; bergabeparameter fr EEPROM Speicherung laden
			lds R1, SchaltwertL
			lds R2, SchaltwertH
			rcall EEPROM_Write	; Schaltwert Magnetventil Aus in EEPROM speichern
			rcall Delay400ms	; nach Tastendruck 400ms warten
			ret

SchaltAnLuft:
			lds rmp, Menuemerker; letzten Menmerker laden, damit nur 1mal der Schaltwert fr
			sbrc rmp, 2			; Luftpumpe An aus dem EEPROM in SRAM geladen werden
			rjmp SchaltAnLuft1
			ldi rmp, 0b00000100
			sts Menuemerker, rmp; nochmaliges Laden aus EEPROM sperren
			ldi ZH, high(WertLuft_An)
			ldi ZL, low(WertLuft_An)
			rcall EEPROM_Read	; aus EEPROM Schaltwert lesen
			sts SchaltwertH, R2
			sts SchaltwertL, R1	; ausgelesenen Schaltwert speichern
SchaltAnLuft1:
			lds R1, SchaltwertL	; Schaltwert aktuell aus SRAM laden zur Vorbereitung anschlieender
			lds R2, SchaltwertH	; Wandlung
			rcall Wandlung		; Schaltwert in Fliekommazahl wandeln
			ldi rmp,0x81 		
			rcall Lcd4Ctrl		; Curser verschieben in 1.Zeile 2.Zeichen in LCD
			ldi ZH, high(text2)
			ldi ZL, low(text2)	; Adresse des text2 von EEPROM laden
			rcall LcdEepPrint	; Unterfunktion LcdEepPrint aufrufen
			ldi rmp,0xC0 		
			rcall Lcd4Ctrl		; Curser verschieben in 2.Zeile 1.Zeichen in LCD
			ldi ZH, high(text5)
			ldi ZL, low(text5)	; Adresse des text5 von EEPROM laden
			rcall LcdEepPrint	; Unterfunktion LcdEepPrint aufrufen
			ldi rmp,0xC9
			rcall Lcd4Ctrl		; Curser verschieben in 2.Zeile 10.Zeichen in LCD
			ldi ZH, high(Zehner)
			ldi ZL, low(Zehner)	; Zeiger auf Adresse fr gewandelte Zahl -> Zehnerstelle setzen
								; Unterschlagung der Zehnerstelle mglich

			ldi rmp, 5			; bergabewert fr die Anzeige der 5 nchst folgenden Zeichen
								; aus SRAM
			rcall Lcd4RTxt
			sbis key_port, 5	; berspringe nchste Zeile, wenn Bit 5 gesetzt ist, da low aktiv
			rcall SchaltAnLuftstore
			rjmp HAUPT	
SchaltAnLuftstore:
			ldi ZH, high(WertLuft_An)
			ldi ZL, low(WertLuft_An) ; bergabeparameter fr EEPROM Speicherung laden
			lds R1, SchaltwertL
			lds R2, SchaltwertH
			rcall EEPROM_Write	; Schaltwert Luftpumpe An in EEPROM speichern
			rcall Delay400ms	; nach Tastendruck 400ms warten
			ret

SchaltAusLuft:
			lds rmp, Menuemerker; letzten Menmerker laden, damit nur 1mal der Schaltwert fr
			sbrc rmp, 3			; Luftpumpe Aus aus dem EEPROM in SRAM geladen werden
			rjmp SchaltAusLuft1
			ldi rmp, 0b00001000
			sts Menuemerker, rmp; nochmaliges Laden aus EEPROM sperren
			ldi ZH, high(WertLuft_Aus)
			ldi ZL, low(WertLuft_Aus)
			rcall EEPROM_Read	; aus EEPROM Schaltwert lesen
			sts SchaltwertH, R2
			sts SchaltwertL, R1	; ausgelesenen Schaltwert speichern
SchaltAusLuft1:
			lds R1, SchaltwertL	; Schaltwert aktuell aus SRAM laden zur Vorbereitung anschlieender
			lds R2, SchaltwertH	; Wandlung
			rcall Wandlung		; Schaltwert in Fliekommazahl wandeln
			ldi rmp,0x81 		
			rcall Lcd4Ctrl		; Curser verschieben in 1.Zeile 2.Zeichen in LCD
			ldi ZH, high(text2)
			ldi ZL, low(text2)	; Adresse des text2 von EEPROM laden
			rcall LcdEepPrint	; Unterfunktion LcdEepPrint aufrufen
			ldi rmp,0xC0 		
			rcall Lcd4Ctrl		; Curser verschieben in 2.Zeile 1.Zeichen in LCD
			ldi ZH, high(text6)
			ldi ZL, low(text6)	; Adresse des text6 von EEPROM laden
			rcall LcdEepPrint	; Unterfunktion LcdEepPrint aufrufen
			ldi rmp,0xC9
			rcall Lcd4Ctrl		; Curser verschieben in 2.Zeile 10.Zeichen in LCD
			ldi ZH, high(Zehner)
			ldi ZL, low(Zehner)	; Zeiger auf Adresse fr gewandelte Zahl -> Zehnerstelle setzen
								; Unterschlagung der Zehnerstelle mglich

			ldi rmp, 5			; bergabewert fr die Anzeige der 5 nchst folgenden Zeichen
								; aus SRAM
			rcall Lcd4RTxt
			sbis key_port, 5	; berspringe nchste Zeile, wenn Bit 5 gesetzt ist, da low aktiv
			rcall SchaltAusLuftstore
			rjmp HAUPT	
SchaltAusLuftstore:
			ldi ZH, high(WertLuft_Aus)
			ldi ZL, low(WertLuft_Aus) ; bergabeparameter fr EEPROM Speicherung laden
			lds R1, SchaltwertL
			lds R2, SchaltwertH
			rcall EEPROM_Write	; Schaltwert Luftpumpe Aus in EEPROM speichern
			rcall Delay400ms	; nach Tastendruck 400ms warten
			ret

.include "LCD_Basisroutine.asm"
.include "LCD-Intro.asm"
.include "Fliesskomma_10bit_ph-Wert.asm"
.include "dcf77dec.asm"

.eseg                   ; EEPROM-Segment aktivieren
text1:
.db "ph-Wert", 0		; Text 1 im EEPROM terminiert mit 0
text2:
.db "ph-Schaltwert", 0	; Text 2 im EEPROM terminiert mit 0
text3:
.db "MV An", 0			; Text 3 im EEPROM terminiert mit 0
text4:
.db "MV Aus", 0			; Text 4 im EEPROM terminiert mit 0
text5:
.db "Luft An", 0		; Text 5 im EEPROM terminiert mit 0
text6:	
.db "Luft Aus", 0		; Text 6 im EEPROM terminiert mit 0
text7:	
.db "aktuell", 0		; Text 7 im EEPROM terminiert mit 0
Reserve:
.db 0x00, 0x00			; Reserveplatz im EEPROM

; Anzeigetexte fr Wochentage dcf77-Modul
WTag1:	
.db "Mo", 0
WTag2:
.db "Di", 0
WTag3:
.db "Mi", 0
WTag4:
.db "Do", 0
WTag5:
.db "Fr", 0
WTag6:
.db "Sa", 0
WTag7:
.db "So", 0
WTagError:
.db "EE", 0 			; Fehleranzeige fr Wochnetag

; 16-bit Festwerte (Highbyte, Lowbyte, Terminierung), Terminierung zur Sicherheit
WertMV_An:
.db 0x01, 0x19, 0		; Startwert (Erstbetrieb) fr Schaltschwelle Magnetventil An
WertMV_Aus:
.db 0x01, 0x00, 0		; Startwert (Erstbetrieb) fr Schaltschwelle Magnetventil Aus
WertLuft_An:
.db 0x00, 0xF9, 0		; Startwert (Erstbetrieb) fr Schaltschwelle Luftpumpe An
WertLuft_Aus:
.db 0x01, 0x09, 0		; Startwert (Erstbetrieb) fr Schaltschwelle Luftpumpe Aus

